-----------------------------------------------------------------------------------------------
-- Client Lua Script for Challenger - By Ciuine - GPLv3
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChallengesLib"
require "GameLib"
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- Challenger Module Definition
-----------------------------------------------------------------------------------------------
local Challenger = {} 
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Challenger:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 


    return o
end

function Challenger:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Challenger"
	local tDependencies = {
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Challenger OnLoad
-----------------------------------------------------------------------------------------------
function Challenger:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Challenger.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "ChallengerForm", nil, self)
	self.wndOpt = Apollo.LoadForm(self.xmlDoc, "Options", nil, self)
	self.tList = {}
	self.tButtons = {}
	self.tActiveChallenges = {}
	self.tColors = {[0] = ApolloColor.new("ffff0000"), [1] = ApolloColor.new("ff00ff00"),[2] = ApolloColor.new("ffff00ff"),
	[3] = ApolloColor.new("ff0000ff"),[4] = ApolloColor.new("ffffff00"),[5] = ApolloColor.new("ff00ffff"),[6] = ApolloColor.new("ffbbaabb"), [7] = ApolloColor.new("ff00bbaa")}
end

-----------------------------------------------------------------------------------------------
-- Challenger OnDocLoaded
-----------------------------------------------------------------------------------------------
function Challenger:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		
	    self.wndMain:Show(true, true)

		Challenger.timer = ApolloTimer.Create(1.0, true, "OnTimer", self)


		Apollo.RegisterSlashCommand("challenger", "OnSlashShow", self)
		Apollo.RegisterSlashCommand("challopt", "OnConfigure", self)
		
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ChallengerConfigure", "OnConfigure", self)
		
		Apollo.RegisterEventHandler("ChallengeFailTime", "ChallengeEnd", self)
	    Apollo.RegisterEventHandler("ChallengeFailArea", "ChallengeEnd", self)
		Apollo.RegisterEventHandler("ChallengeActivate", "ChallengeStart", self)
		Apollo.RegisterEventHandler("ChallengeFailGeneric", "ChallengeEnd", self)
		
		Apollo.RegisterEventHandler("ChallengeAbandon", "ChallengeIdEnd", self)
		Apollo.RegisterEventHandler("ChallengeCompleted", "ChallengeIdEnd", self)
	end
end

function Challenger:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	local tSavedData = {}
	tSavedData.tAnchorOffsets = {self.wndMain:GetAnchorOffsets()}
	tSavedData.tOptAnchors = {self.wndOpt:GetAnchorOffsets()}
	tSavedData.bShowCooldowns = Challenger.bShowCooldowns
	tSavedData.nHideRange = Challenger.nHideRange
	tSavedData.bHideRange = Challenger.bHideRange
	tSavedData.bHideGlobals = Challenger.bHideGlobals
	tSavedData.tDeletedChallenges = Challenger.tDeletedChallenges
	tSavedData.bMetaToMove = Challenger.bMetaToMove
	tSavedData.bShowDistance = Challenger.bShowDistance
	return tSavedData
end

function Challenger:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	
	if tSavedData.tAnchorOffsets then
		self.wndMain:SetAnchorOffsets(unpack(tSavedData.tAnchorOffsets))
	end
	if tSavedData.tOptAnchors then
		self.wndOpt:SetAnchorOffsets(unpack(tSavedData.tOptAnchors))
	end
	Challenger.bShowCooldowns = tSavedData.bShowCooldowns or false
	Challenger.bHideRange = tSavedData.bHideRange or false
	Challenger.bHideGlobals = tSavedData.bHideGlobals or true
	Challenger.nHideRange = tSavedData.nHideRange or 200
	Challenger.bMetaToMove = tSavedData.bMetaToMove or false
	Challenger.tDeletedChallenges = tSavedData.tDeletedChallenges or {}
	Challenger.bShowDistance = tSavedData.bShowDistance or false
	self.wndOpt:FindChild("Cooldowns"):SetCheck(Challenger.bShowCooldowns)
	self.wndOpt:FindChild("Range"):SetCheck(Challenger.bHideRange)
	self.wndOpt:FindChild("Globals"):SetCheck(Challenger.bHideGlobals)
	self.wndOpt:FindChild("Meta"):SetCheck(Challenger.bMetaToMove)
	self.wndOpt:FindChild("RangeBox"):SetText(Challenger.nHideRange)
	self.wndMain:SetStyle("RequireMetaKeyToMove", Challenger.bMetaToMove)
	self.wndOpt:FindChild("Distance"):SetCheck(Challenger.bShowDistance)
end

function Challenger:OnSlashShow()
	self.wndMain:Show(not self.wndMain:IsVisible())
end

function Challenger:OnConfigure()
	self.wndOpt:Show(not self.wndOpt:IsVisible())
end

function Challenger:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Challenger", {"ChallengerConfigure", "", ""})
end

function Challenger:ChallengeEnd(uChallenge)
	self.tActiveChallenges[uChallenge:GetType()] = nil
end

function Challenger:ChallengeIdEnd(nChallenge)
	for i, v in pairs(self.tActiveChallenges) do
		if tonumber(v) == nChallenge then
			self.tActiveChallenges[i] = nil
		end
	end
end

function Challenger:ChallengeStart(uChallenge)
	self.tActiveChallenges[uChallenge:GetType()] = uChallenge:GetId()
end

-----------------------------------------------------------------------------------------------
-- Challenger Functions
-----------------------------------------------------------------------------------------------
local function SortTableByDist(a, b)
	return a:GetData().tDist < b:GetData().tDist
end

function Challenger:OnTimer()
	local bShowWindow = false
	if GameLib:GetPlayerUnit() and ChallengesLib.GetActiveChallengeList() ~= nil then
		for i, v in pairs(ChallengesLib.GetActiveChallengeList()) do
			local showChallenge = true
			local challengeId = v:GetId()
			if Challenger.tDeletedChallenges ~= nil then
				for j, x in pairs(Challenger.tDeletedChallenges) do
					if x == challengeId then
						showChallenge = false
					end
				end
			end
			local challengePos = v:GetMapLocation()
			local playerPos = GameLib:GetPlayerUnit():GetPosition()
			local distToP = 10000
			local distText = ""
			local compCount = v:GetCompletionCount()
			local compTotal = v:GetCompletionTotal()
			local zoneRestrict = v:GetZoneRestrictionInfo()
			local startLoc = v:GetStartLocationRestrictionId()
			local eType = v:GetType()
			local isInCooldown = v:IsInCooldown()
			local isActive = v:IsActivated()
			local challengeName = v:GetName()
			if challengePos ~= nil then
				distToP = math.floor(math.sqrt((playerPos.x - challengePos.x)^2 + (playerPos.y - challengePos.y)^2 + (playerPos.z - challengePos.z)^2))
				if Challenger.bShowDistance then
					distText = " " .. distToP .. "m"
					if Challenger.nHideRange == nil then
						Challenger.nHideRange = 200
					end
				end
			end
			if showChallenge and (compCount < compTotal or compTotal == -1) and
			(GameLib.IsInWorldZone(zoneRestrict.idSubZone) or (zoneRestrict.idSubZone == 0 and not Challenger.bHideGlobals)) and
			(GameLib.IsInLocation(startLoc) or startLoc == 0) and 
			not isActive and not isInCooldown and ((Challenger.bHideRange and Challenger.nHideRange and distToP <= Challenger.nHideRange) or not Challenger.bHideRange)
			then
				self.tList[challengeId] = true
				bShowWindow = true
				if self.tButtons[challengeId] == nil then
					local newWindow = Apollo.LoadForm(self.xmlDoc, "InnerChallenger", self.wndMain, self)
					newWindow:Show(true)
					newWindow:SetText(challengeName)
					newWindow:FindChild("Color"):SetBGColor(self.tColors[eType])
					self.tButtons[challengeId] = newWindow
				end
				if self.tButtons[challengeId] then
					self.tButtons[challengeId]:SetData({uChallenge = v, tDist = distToP})
					self.tButtons[challengeId]:SetText(challengeName .. distText)
				end
			elseif showChallenge and Challenger.bShowCooldowns and (compCount < compTotal or compTotal == -1) and
			(GameLib.IsInWorldZone(zoneRestrict.idSubZone) or (zoneRestrict.idSubZone == 0 and not Challenger.bHideGlobals)) and
			(GameLib.IsInLocation(startLoc) or startLoc == 0) and 
			not isActive and isInCooldown and ((Challenger.bHideRange and distToP <= Challenger.nHideRange) or not Challenger.bHideRange) then
				self.tList[challengeId] = true
				bShowWindow = true
				if self.tButtons[challengeId] == nil then
					local newWindow = Apollo.LoadForm(self.xmlDoc, "InnerChallenger", self.wndMain, self)
					newWindow:SetNormalTextColor(ApolloColor.new("ffff0000"))
					newWindow:SetFlybyTextColor(ApolloColor.new("ffff0000"))
					newWindow:SetText(challengeName .. " " .. v:GetTimeStr())
					newWindow:Show(true)
					newWindow:FindChild("Color"):SetBGColor(self.tColors[eType])
					self.tButtons[challengeId] = newWindow
				end
				if self.tButtons[challengeId] then
					self.tButtons[challengeId]:SetData({uChallenge = v, tDist = distToP})
					self.tButtons[challengeId]:SetText(challengeName .. distText .. " " .. v:GetTimeStr())
				end
			else
				self.tListchallengeId = false
				if self.tButtons[i] then
					self.tButtons[i]:Destroy()
					self.tButtons[i] = nil
				end
				if v:IsActivated() then
					self.tActiveChallenges[eType] = challengeId
				end
			end 
		end
		self.wndMain:ArrangeChildrenVert(0, SortTableByDist)
		self.wndMain:Show(bShowWindow)
	end
end

-----------------------------------------------------------------------------------------------
-- ChallengerForm Functions
-----------------------------------------------------------------------------------------------
function Challenger:StartChallengeBtn(wndControl, wndHandler, eMouseButton)
	if eMouseButton == 1 then table.insert(Challenger.tDeletedChallenges, wndControl:GetData().uChallenge:GetId()) return end
	if wndControl:GetData().uChallenge:IsInCooldown() then Print("Challenge is on cooldown.") return end
	if self.tActiveChallenges[wndControl:GetData().uChallenge:GetType()] ~= nil then Print("Challenge type already active.") return end
	ChallengesLib.ActivateChallenge(wndControl:GetData().uChallenge:GetId())
	if wndControl:GetData().uChallenge:IsActivated() then
		self.tActiveChallenges[wndControl:GetData().uChallenge:GetType()] = wndControl:GetData().uChallenge:GetId()
	end
end
---------------------------------------------------------------------------------------------------
-- Options Functions
---------------------------------------------------------------------------------------------------
function Challenger:ShowCooldowns( wndHandler, wndControl )
	Challenger.bShowCooldowns = wndControl:IsChecked()
end

function Challenger:RangeChange( wndHandler, wndControl, strText )
	if tonumber(strText) and tonumber(strText) ~= nil then
		Challenger.nHideRange = tonumber(strText)
	end
end

function Challenger:ShowDistance( wndHandler, wndControl )
	Challenger.bShowDistance = wndControl:IsChecked()
end

function Challenger:HideRange( wndHandler, wndControl )
	Challenger.bHideRange = wndControl:IsChecked()
end

function Challenger:HideGlobals( wndHandler, wndControl )
	Challenger.bHideGlobals = wndControl:IsChecked()
end

function Challenger:ResetDeletedChallenges( )
	Challenger.tDeletedChallenges = {}
end

function Challenger:MetaToMove( wndHandler, wndControl )
	Challenger.bMetaToMove = wndControl:IsChecked()
	self.wndMain:SetStyle("RequireMetaKeyToMove", wndControl:IsChecked())
end

-----------------------------------------------------------------------------------------------
-- Challenger Instance
-----------------------------------------------------------------------------------------------
local ChallengerInst = Challenger:new()
ChallengerInst:Init()
